@@ -12,24 +12,7 @@ if TYPE_CHECKING:
     from docutils.nodes import Element
 
     from sphinx.application import Sphinx
-
-# deprecated name -> (object to return, canonical path or empty string)
-_DEPRECATED_OBJECTS = {
-    'meta': (nodes.meta, 'docutils.nodes.meta'),  # type: ignore[attr-defined]
-    'docutils_meta': (nodes.meta, 'docutils.nodes.meta'),  # type: ignore[attr-defined]
-}
-
-
-def __getattr__(name):
-    if name not in _DEPRECATED_OBJECTS:
-        msg = f'module {__name__!r} has no attribute {name!r}'
-        raise AttributeError(msg)
-
-    from sphinx.deprecation import _deprecation_warning
-
-    deprecated_object, canonical_name = _DEPRECATED_OBJECTS[name]
-    _deprecation_warning(__name__, name, canonical_name, remove=(7, 0))
-    return deprecated_object
+    from sphinx.util.typing import ExtensionMetadata
 
 
 class document(nodes.document):
@@ -44,7 +27,7 @@ class document(nodes.document):
 
     def set_id(self, node: Element, msgnode: Element | None = None,
                suggested_prefix: str = '') -> str:
-        return super().set_id(node, msgnode, suggested_prefix)  # type: ignore[call-arg]
+        return super().set_id(node, msgnode, suggested_prefix)
 
 
 class translatable(nodes.Node):
@@ -79,6 +62,7 @@ class translatable(nodes.Node):
 
 class not_smartquotable:
     """A node which does not support smart-quotes."""
+
     support_smartquotes = False
 
 
@@ -87,7 +71,7 @@ class toctree(nodes.General, nodes.Element, translatable):
 
     def preserve_original_messages(self) -> None:
         # toctree entries
-        rawentries = self.setdefault('rawentries', [])
+        rawentries: list[str] = self.setdefault('rawentries', [])
         for title, _docname in self['entries']:
             if title:
                 rawentries.append(title)
@@ -163,11 +147,12 @@ class desc_signature(_desc_classes_injector, nodes.Part, nodes.Inline, nodes.Tex
 
     This node always has the classes ``sig``, ``sig-object``, and the domain it belongs to.
     """
+
     # Note: the domain name is being added through a post-transform DescSigAddDomainAsClass
     classes = ['sig', 'sig-object']
 
     @property
-    def child_text_separator(self):
+    def child_text_separator(self) -> str:  # type: ignore[override]
         if self.get('is_multiline'):
             return ' '
         else:
@@ -181,6 +166,7 @@ class desc_signature_line(nodes.Part, nodes.Inline, nodes.FixedTextElement):
     with ``is_multiline`` set to ``True``.
     Set ``add_permalink = True`` for the line that should get the permalink.
     """
+
     sphinx_line_type = ''
 
 
@@ -199,6 +185,7 @@ class desc_inline(_desc_classes_injector, nodes.Inline, nodes.TextElement):
     This node always has the classes ``sig``, ``sig-inline``,
     and the name of the domain it belongs to.
     """
+
     classes = ['sig', 'sig-inline']
 
     def __init__(self, domain: str, *args: Any, **kwargs: Any) -> None:
@@ -219,6 +206,7 @@ class desc_name(_desc_classes_injector, nodes.Part, nodes.Inline, nodes.FixedTex
 
     This node always has the class ``sig-name``.
     """
+
     classes = ['sig-name', 'descname']  # 'descname' is for backwards compatibility
 
 
@@ -230,6 +218,7 @@ class desc_addname(_desc_classes_injector, nodes.Part, nodes.Inline, nodes.Fixed
 
     This node always has the class ``sig-prename``.
     """
+
     # 'descclassname' is for backwards compatibility
     classes = ['sig-prename', 'descclassname']
 
@@ -256,9 +245,10 @@ class desc_parameterlist(nodes.Part, nodes.Inline, nodes.FixedTextElement):
     Set ``multi_line_parameter_list = True`` to describe a multi-line parameter list.
     In that case each parameter will then be written on its own, indented line.
     """
+
     child_text_separator = ', '
 
-    def astext(self):
+    def astext(self) -> str:
         return f'({super().astext()})'
 
 
@@ -269,9 +259,10 @@ class desc_type_parameter_list(nodes.Part, nodes.Inline, nodes.FixedTextElement)
     Set ``multi_line_parameter_list = True`` to describe a multi-line type parameters list.
     In that case each type parameter will then be written on its own, indented line.
     """
+
     child_text_separator = ', '
 
-    def astext(self):
+    def astext(self) -> str:
         return f'[{super().astext()}]'
 
 
@@ -285,6 +276,7 @@ class desc_type_parameter(nodes.Part, nodes.Inline, nodes.FixedTextElement):
 
 class desc_optional(nodes.Part, nodes.Inline, nodes.FixedTextElement):
     """Node for marking optional parts of the parameter list."""
+
     child_text_separator = ', '
 
     def astext(self) -> str:
@@ -315,6 +307,7 @@ SIG_ELEMENTS: set[type[desc_sig_element]] = set()
 
 class desc_sig_element(nodes.inline, _desc_classes_injector):
     """Common parent class of nodes for inline text of a signature."""
+
     classes: list[str] = []
 
     def __init__(self, rawsource: str = '', text: str = '',
@@ -322,7 +315,7 @@ class desc_sig_element(nodes.inline, _desc_classes_injector):
         super().__init__(rawsource, text, *children, **attributes)
         self['classes'].extend(self.classes)
 
-    def __init_subclass__(cls, *, _sig_element=False, **kwargs):
+    def __init_subclass__(cls, *, _sig_element: bool = False, **kwargs: Any) -> None:
         super().__init_subclass__(**kwargs)
         if _sig_element:
             # add the class to the SIG_ELEMENTS set if asked
@@ -334,6 +327,7 @@ class desc_sig_element(nodes.inline, _desc_classes_injector):
 
 class desc_sig_space(desc_sig_element, _sig_element=True):
     """Node for a space in a signature."""
+
     classes = ["w"]
 
     def __init__(self, rawsource: str = '', text: str = ' ',
@@ -343,41 +337,49 @@ class desc_sig_space(desc_sig_element, _sig_element=True):
 
 class desc_sig_name(desc_sig_element, _sig_element=True):
     """Node for an identifier in a signature."""
+
     classes = ["n"]
 
 
 class desc_sig_operator(desc_sig_element, _sig_element=True):
     """Node for an operator in a signature."""
+
     classes = ["o"]
 
 
 class desc_sig_punctuation(desc_sig_element, _sig_element=True):
     """Node for punctuation in a signature."""
+
     classes = ["p"]
 
 
 class desc_sig_keyword(desc_sig_element, _sig_element=True):
     """Node for a general keyword in a signature."""
+
     classes = ["k"]
 
 
 class desc_sig_keyword_type(desc_sig_element, _sig_element=True):
     """Node for a keyword which is a built-in type in a signature."""
+
     classes = ["kt"]
 
 
 class desc_sig_literal_number(desc_sig_element, _sig_element=True):
     """Node for a numeric literal in a signature."""
+
     classes = ["m"]
 
 
 class desc_sig_literal_string(desc_sig_element, _sig_element=True):
     """Node for a string literal in a signature."""
+
     classes = ["s"]
 
 
 class desc_sig_literal_char(desc_sig_element, _sig_element=True):
     """Node for a character literal in a signature."""
+
     classes = ["sc"]
 
 
@@ -387,8 +389,8 @@ class desc_sig_literal_char(desc_sig_element, _sig_element=True):
 class versionmodified(nodes.Admonition, nodes.TextElement):
     """Node for version change entries.
 
-    Currently used for "versionadded", "versionchanged" and "deprecated"
-    directives.
+    Currently used for "versionadded", "versionchanged", "deprecated"
+    and "versionremoved" directives.
     """
 
 
@@ -425,7 +427,7 @@ class index(nodes.Invisible, nodes.Inline, nodes.TextElement):
 
 
 class centered(nodes.Part, nodes.TextElement):
-    """This node is deprecated."""
+    """Deprecated."""
 
 
 class acks(nodes.Element):
@@ -479,6 +481,7 @@ class pending_xref(nodes.Inline, nodes.Element):
     These nodes are resolved before writing output, in
     BuildEnvironment.resolve_references.
     """
+
     child_text_separator = ''
 
 
@@ -553,7 +556,7 @@ class manpage(nodes.Inline, nodes.FixedTextElement):
     """Node for references to manpages."""
 
 
-def setup(app: Sphinx) -> dict[str, Any]:
+def setup(app: Sphinx) -> ExtensionMetadata:
     app.add_node(toctree)
 
     app.add_node(desc)
