@@ -8,7 +8,7 @@ import os
 import sys
 import time
 from os import path
-from typing import TYPE_CHECKING, Any, Callable
+from typing import TYPE_CHECKING, Any
 
 # try to import readline, unix specific enhancement
 try:
@@ -31,10 +31,13 @@ from docutils.utils import column_width
 import sphinx.locale
 from sphinx import __display_version__, package_dir
 from sphinx.locale import __
-from sphinx.util.console import bold, color_terminal, colorize, nocolor, red  # type: ignore
+from sphinx.util.console import bold, color_terminal, colorize, nocolor, red
 from sphinx.util.osutil import ensuredir
 from sphinx.util.template import SphinxRenderer
 
+if TYPE_CHECKING:
+    from collections.abc import Callable, Sequence
+
 EXTENSIONS = {
     'autodoc': __('automatically insert docstrings from modules'),
     'doctest': __('automatically test code snippets in doctest blocks'),
@@ -202,7 +205,6 @@ def ask_user(d: dict[str, Any]) -> None:
     * makefile:  make Makefile
     * batchfile: make command file
     """
-
     print(bold(__('Welcome to the Sphinx %s quickstart utility.')) % __display_version__)
     print()
     print(__('Please enter values for the following settings (just press Enter to\n'
@@ -237,9 +239,9 @@ def ask_user(d: dict[str, Any]) -> None:
 
     if 'dot' not in d:
         print()
-        print(__('Inside the root directory, two more directories will be created; "_templates"\n'      # noqa: E501
-                 'for custom HTML templates and "_static" for custom stylesheets and other static\n'    # noqa: E501
-                 'files. You can enter another prefix (such as ".") to replace the underscore.'))       # noqa: E501
+        print(__('Inside the root directory, two more directories will be created; "_templates"\n'      # NoQA: E501
+                 'for custom HTML templates and "_static" for custom stylesheets and other static\n'    # NoQA: E501
+                 'files. You can enter another prefix (such as ".") to replace the underscore.'))       # NoQA: E501
         d['dot'] = do_prompt(__('Name prefix for templates and static dir'), '_', ok)
 
     if 'project' not in d:
@@ -324,7 +326,10 @@ def ask_user(d: dict[str, Any]) -> None:
 
 
 def generate(
-    d: dict, overwrite: bool = True, silent: bool = False, templatedir: str | None = None,
+    d: dict[str, Any],
+    overwrite: bool = True,
+    silent: bool = False,
+    templatedir: str | None = None,
 ) -> None:
     """Generate project based on values in *d*."""
     template = QuickstartRenderer(templatedir or '')
@@ -370,29 +375,25 @@ def generate(
             if 'quiet' not in d:
                 print(__('File %s already exists, skipping.') % fpath)
 
-    conf_path = os.path.join(templatedir, 'conf.py_t') if templatedir else None
+    conf_path = os.path.join(templatedir, 'conf.py.jinja') if templatedir else None
     if not conf_path or not path.isfile(conf_path):
-        conf_path = os.path.join(package_dir, 'templates', 'quickstart', 'conf.py_t')
+        conf_path = os.path.join(package_dir, 'templates', 'quickstart', 'conf.py.jinja')
     with open(conf_path, encoding="utf-8") as f:
         conf_text = f.read()
 
     write_file(path.join(srcdir, 'conf.py'), template.render_string(conf_text, d))
 
     masterfile = path.join(srcdir, d['master'] + d['suffix'])
-    if template._has_custom_template('quickstart/master_doc.rst_t'):
-        msg = ('A custom template `master_doc.rst_t` found. It has been renamed to '
-               '`root_doc.rst_t`.  Please rename it on your project too.')
+    if template._has_custom_template('quickstart/master_doc.rst.jinja'):
+        msg = ('A custom template `master_doc.rst.jinja` found. It has been renamed to '
+               '`root_doc.rst.jinja`.  Please rename it on your project too.')
         print(colorize('red', msg))
-        write_file(masterfile, template.render('quickstart/master_doc.rst_t', d))
+        write_file(masterfile, template.render('quickstart/master_doc.rst.jinja', d))
     else:
-        write_file(masterfile, template.render('quickstart/root_doc.rst_t', d))
+        write_file(masterfile, template.render('quickstart/root_doc.rst.jinja', d))
 
-    if d.get('make_mode') is True:
-        makefile_template = 'quickstart/Makefile.new_t'
-        batchfile_template = 'quickstart/make.bat.new_t'
-    else:
-        makefile_template = 'quickstart/Makefile_t'
-        batchfile_template = 'quickstart/make.bat_t'
+    makefile_template = 'quickstart/Makefile.new.jinja'
+    batchfile_template = 'quickstart/make.bat.new.jinja'
 
     if d['makefile'] is True:
         d['rsrcdir'] = 'source' if d['sep'] else '.'
@@ -425,7 +426,7 @@ def generate(
     print()
 
 
-def valid_dir(d: dict) -> bool:
+def valid_dir(d: dict[str, Any]) -> bool:
     dir = d['path']
     if not path.exists(dir):
         return True
@@ -448,10 +449,7 @@ def valid_dir(d: dict) -> bool:
         d['dot'] + 'templates',
         d['master'] + d['suffix'],
     ]
-    if set(reserved_names) & set(os.listdir(dir)):
-        return False
-
-    return True
+    return not set(reserved_names) & set(os.listdir(dir))
 
 
 def get_parser() -> argparse.ArgumentParser:
@@ -521,12 +519,10 @@ def get_parser() -> argparse.ArgumentParser:
     group.add_argument('--no-batchfile', action='store_false',
                        dest='batchfile',
                        help=__('do not create batchfile'))
+    # --use-make-mode is a no-op from Sphinx 8.
     group.add_argument('-m', '--use-make-mode', action='store_true',
                        dest='make_mode', default=True,
                        help=__('use make-mode for Makefile/make.bat'))
-    group.add_argument('-M', '--no-use-make-mode', action='store_false',
-                       dest='make_mode',
-                       help=__('do not use make-mode for Makefile/make.bat'))
 
     group = parser.add_argument_group(__('Project templating'))
     group.add_argument('-t', '--templatedir', metavar='TEMPLATEDIR',
@@ -539,7 +535,7 @@ def get_parser() -> argparse.ArgumentParser:
     return parser
 
 
-def main(argv: list[str] = sys.argv[1:]) -> int:
+def main(argv: Sequence[str] = (), /) -> int:
     locale.setlocale(locale.LC_ALL, '')
     sphinx.locale.init_console()
 
@@ -549,7 +545,7 @@ def main(argv: list[str] = sys.argv[1:]) -> int:
     # parse options
     parser = get_parser()
     try:
-        args = parser.parse_args(argv)
+        args = parser.parse_args(argv or sys.argv[1:])
     except SystemExit as err:
         return err.code  # type: ignore[return-value]
 
@@ -605,4 +601,4 @@ def main(argv: list[str] = sys.argv[1:]) -> int:
 
 
 if __name__ == '__main__':
-    raise SystemExit(main())
+    raise SystemExit(main(sys.argv[1:]))
