@@ -20,6 +20,7 @@ if TYPE_CHECKING:
     from sphinx.application import Sphinx
     from sphinx.builders import Builder
     from sphinx.environment import BuildEnvironment
+    from sphinx.util.typing import ExtensionMetadata
 
 
 logger = logging.getLogger(__name__)
@@ -34,6 +35,7 @@ class MathReferenceRole(XRefRole):
 
 class MathDomain(Domain):
     """Mathematics domain."""
+
     name = 'math'
     label = 'mathematics'
 
@@ -58,8 +60,8 @@ class MathDomain(Domain):
     def note_equation(self, docname: str, labelid: str, location: Any = None) -> None:
         if labelid in self.equations:
             other = self.equations[labelid][0]
-            logger.warning(__('duplicate label of equation %s, other instance in %s') %
-                           (labelid, other), location=location)
+            logger.warning(__('duplicate label of equation %s, other instance in %s'),
+                           labelid, other, location=location)
 
         self.equations[labelid] = (docname, self.env.new_serialno('eqno') + 1)
 
@@ -72,7 +74,7 @@ class MathDomain(Domain):
     def process_doc(self, env: BuildEnvironment, docname: str,
                     document: nodes.document) -> None:
         def math_node(node: Node) -> bool:
-            return isinstance(node, (nodes.math, nodes.math_block))
+            return isinstance(node, nodes.math | nodes.math_block)
 
         self.data['has_equations'][docname] = any(document.findall(math_node))
 
@@ -104,6 +106,7 @@ class MathDomain(Domain):
                 if docname in env.toc_fignumbers:
                     numbers = env.toc_fignumbers[docname]['displaymath'].get(node_id, ())
                     eqno = '.'.join(map(str, numbers))
+                    eqno = env.config.math_numsep.join(eqno.rsplit('.', 1))
                 else:
                     eqno = ''
             else:
@@ -134,13 +137,16 @@ class MathDomain(Domain):
         return []
 
     def has_equations(self, docname: str | None = None) -> bool:
-        if docname:
-            return self.data['has_equations'].get(docname, False)
-        else:
+        if not docname:
             return any(self.data['has_equations'].values())
 
+        return (
+            self.data['has_equations'].get(docname, False)
+            or any(map(self.has_equations, self.env.toctree_includes.get(docname, ())))
+        )
 
-def setup(app: Sphinx) -> dict[str, Any]:
+
+def setup(app: Sphinx) -> ExtensionMetadata:
     app.add_domain(MathDomain)
     app.add_role('eq', MathReferenceRole(warn_dangling=True))
 
