@@ -4,19 +4,23 @@ from __future__ import annotations
 
 import os
 import re
+from hashlib import sha1
 from math import ceil
-from typing import Any
+from typing import TYPE_CHECKING, Any
 
 from docutils import nodes
 
-from sphinx.application import Sphinx
 from sphinx.locale import __
 from sphinx.transforms import SphinxTransform
-from sphinx.util import logging, requests, sha1
+from sphinx.util import logging, requests
 from sphinx.util.http_date import epoch_to_rfc1123, rfc1123_to_epoch
 from sphinx.util.images import get_image_extension, guess_mimetype, parse_data_uri
 from sphinx.util.osutil import ensuredir
 
+if TYPE_CHECKING:
+    from sphinx.application import Sphinx
+    from sphinx.util.typing import ExtensionMetadata
+
 logger = logging.getLogger(__name__)
 
 MAX_FILENAME_LEN = 32
@@ -44,7 +48,7 @@ class ImageDownloader(BaseImageConverter):
     default_priority = 100
 
     def match(self, node: nodes.image) -> bool:
-        if self.app.builder.supported_image_types == []:
+        if not self.app.builder.supported_image_types:
             return False
         if self.app.builder.supported_remote_images:
             return False
@@ -57,25 +61,27 @@ class ImageDownloader(BaseImageConverter):
                 basename = basename.split('?')[0]
             if basename == '' or len(basename) > MAX_FILENAME_LEN:
                 filename, ext = os.path.splitext(node['uri'])
-                basename = sha1(filename.encode()).hexdigest() + ext
-            basename = re.sub(CRITICAL_PATH_CHAR_RE, "_", basename)
+                basename = sha1(filename.encode(), usedforsecurity=False).hexdigest() + ext
+            basename = CRITICAL_PATH_CHAR_RE.sub("_", basename)
 
-            dirname = node['uri'].replace('://', '/').translate({ord("?"): "/",
-                                                                 ord("&"): "/"})
-            if len(dirname) > MAX_FILENAME_LEN:
-                dirname = sha1(dirname.encode()).hexdigest()
-            ensuredir(os.path.join(self.imagedir, dirname))
-            path = os.path.join(self.imagedir, dirname, basename)
+            uri_hash = sha1(node['uri'].encode(), usedforsecurity=False).hexdigest()
+            ensuredir(os.path.join(self.imagedir, uri_hash))
+            path = os.path.join(self.imagedir, uri_hash, basename)
 
             headers = {}
             if os.path.exists(path):
                 timestamp: float = ceil(os.stat(path).st_mtime)
                 headers['If-Modified-Since'] = epoch_to_rfc1123(timestamp)
 
-            r = requests.get(node['uri'], headers=headers)
+            config = self.app.config
+            r = requests.get(
+                node['uri'], headers=headers,
+                _user_agent=config.user_agent,
+                _tls_info=(config.tls_verify, config.tls_cacerts),
+            )
             if r.status_code >= 400:
-                logger.warning(__('Could not fetch remote image: %s [%d]') %
-                               (node['uri'], r.status_code))
+                logger.warning(__('Could not fetch remote image: %s [%d]'),
+                               node['uri'], r.status_code)
             else:
                 self.app.env.original_image_uri[path] = node['uri']
 
@@ -92,7 +98,7 @@ class ImageDownloader(BaseImageConverter):
                 if mimetype != '*' and os.path.splitext(basename)[1] == '':
                     # append a suffix if URI does not contain suffix
                     ext = get_image_extension(mimetype)
-                    newpath = os.path.join(self.imagedir, dirname, basename + ext)
+                    newpath = os.path.join(self.imagedir, uri_hash, basename + ext)
                     os.replace(path, newpath)
                     self.app.env.original_image_uri.pop(path)
                     self.app.env.original_image_uri[newpath] = node['uri']
@@ -102,21 +108,20 @@ class ImageDownloader(BaseImageConverter):
                 node['uri'] = path
                 self.app.env.images.add_file(self.env.docname, path)
         except Exception as exc:
-            logger.warning(__('Could not fetch remote image: %s [%s]') % (node['uri'], exc))
+            logger.warning(__('Could not fetch remote image: %s [%s]'), node['uri'], exc)
 
 
 class DataURIExtractor(BaseImageConverter):
     default_priority = 150
 
     def match(self, node: nodes.image) -> bool:
-        if self.app.builder.supported_remote_images == []:
-            return False
         if self.app.builder.supported_data_uri_images is True:
-            return False
+            return False  # do not transform the image; data URIs are valid in the build output
         return node['uri'].startswith('data:')
 
     def handle(self, node: nodes.image) -> None:
         image = parse_data_uri(node['uri'])
+        assert image is not None
         ext = get_image_extension(image.mimetype)
         if ext is None:
             logger.warning(__('Unknown image format: %s...'), node['uri'][:32],
@@ -124,7 +129,7 @@ class DataURIExtractor(BaseImageConverter):
             return
 
         ensuredir(os.path.join(self.imagedir, 'embeded'))
-        digest = sha1(image.data).hexdigest()
+        digest = sha1(image.data, usedforsecurity=False).hexdigest()
         path = os.path.join(self.imagedir, 'embeded', digest + ext)
         self.app.env.original_image_uri[path] = node['uri']
 
@@ -139,8 +144,8 @@ class DataURIExtractor(BaseImageConverter):
 
 def get_filename_for(filename: str, mimetype: str) -> str:
     basename = os.path.basename(filename)
-    basename = re.sub(CRITICAL_PATH_CHAR_RE, "_", basename)
-    return os.path.splitext(basename)[0] + get_image_extension(mimetype)
+    basename = CRITICAL_PATH_CHAR_RE.sub("_", basename)
+    return os.path.splitext(basename)[0] + (get_image_extension(mimetype) or '')
 
 
 class ImageConverter(BaseImageConverter):
@@ -164,6 +169,7 @@ class ImageConverter(BaseImageConverter):
     3. Register your image converter to Sphinx using
        :py:meth:`.Sphinx.add_post_transform`
     """
+
     default_priority = 200
 
     #: The converter is available or not.  Will be filled at the first call of
@@ -184,9 +190,6 @@ class ImageConverter(BaseImageConverter):
     #:     ]
     conversion_rules: list[tuple[str, str]] = []
 
-    def __init__(self, *args: Any, **kwargs: Any) -> None:
-        super().__init__(*args, **kwargs)
-
     def match(self, node: nodes.image) -> bool:
         if not self.app.builder.supported_image_types:
             return False
@@ -202,8 +205,12 @@ class ImageConverter(BaseImageConverter):
         if not self.available:
             return False
         else:
-            rule = self.get_conversion_rule(node)
-            return bool(rule)
+            try:
+                self.get_conversion_rule(node)
+            except ValueError:
+                return False
+            else:
+                return True
 
     def get_conversion_rule(self, node: nodes.image) -> tuple[str, str]:
         for candidate in self.guess_mimetypes(node):
@@ -212,17 +219,21 @@ class ImageConverter(BaseImageConverter):
                 if rule in self.conversion_rules:
                     return rule
 
-        return None
+        msg = 'No conversion rule found'
+        raise ValueError(msg)
 
     def is_available(self) -> bool:
         """Return the image converter is available or not."""
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def guess_mimetypes(self, node: nodes.image) -> list[str]:
+        # The special key ? is set for nonlocal URIs.
         if '?' in node['candidates']:
             return []
         elif '*' in node['candidates']:
-            return [guess_mimetype(node['uri'])]
+            path = os.path.join(self.app.srcdir, node['uri'])
+            guessed = guess_mimetype(path)
+            return [guessed] if guessed is not None else []
         else:
             return node['candidates'].keys()
 
@@ -256,10 +267,10 @@ class ImageConverter(BaseImageConverter):
         *_from* is a path of the source image file, and *_to* is a path
         of the destination file.
         """
-        raise NotImplementedError()
+        raise NotImplementedError
 
 
-def setup(app: Sphinx) -> dict[str, Any]:
+def setup(app: Sphinx) -> ExtensionMetadata:
     app.add_post_transform(ImageDownloader)
     app.add_post_transform(DataURIExtractor)
 
