@@ -1,22 +1,16 @@
-"""
-    sphinx.transforms.references
-    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+"""Docutils transforms used by Sphinx."""
 
-    Docutils transforms used by Sphinx.
+from __future__ import annotations
 
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
-
-from typing import Any, Dict
+from typing import TYPE_CHECKING, Any
 
 from docutils.transforms.references import DanglingReferences
 
 from sphinx.transforms import SphinxTransform
 
-if False:
-    # For type annotation
+if TYPE_CHECKING:
     from sphinx.application import Sphinx
+    from sphinx.util.typing import ExtensionMetadata
 
 
 class SphinxDanglingReferences(DanglingReferences):
@@ -29,13 +23,14 @@ class SphinxDanglingReferences(DanglingReferences):
 
             # suppress INFO level messages for a while
             reporter.report_level = max(reporter.WARNING_LEVEL, reporter.report_level)
-            super().apply()
+            super().apply()  # type: ignore[no-untyped-call]
         finally:
             reporter.report_level = report_level
 
 
 class SphinxDomains(SphinxTransform):
     """Collect objects to Sphinx domains for cross references."""
+
     default_priority = 850
 
     def apply(self, **kwargs: Any) -> None:
@@ -43,7 +38,7 @@ class SphinxDomains(SphinxTransform):
             domain.process_doc(self.env, self.env.docname, self.document)
 
 
-def setup(app: "Sphinx") -> Dict[str, Any]:
+def setup(app: Sphinx) -> ExtensionMetadata:
     app.add_transform(SphinxDanglingReferences)
     app.add_transform(SphinxDomains)
 
