@@ -1,36 +1,43 @@
 """Utility functions common to the C and C++ domains."""
 
+from __future__ import annotations
+
 import re
 from copy import deepcopy
-from typing import Any, Callable, List, Match, Optional, Pattern, Tuple, Union
+from typing import TYPE_CHECKING
 
 from docutils import nodes
-from docutils.nodes import TextElement
 
 from sphinx import addnodes
-from sphinx.config import Config
 from sphinx.util import logging
 
+if TYPE_CHECKING:
+    from collections.abc import Callable, Sequence
+    from typing import Any, TypeAlias
+
+    from docutils.nodes import TextElement
+
+    from sphinx.config import Config
+
+    StringifyTransform: TypeAlias = Callable[[Any], str]
+
 logger = logging.getLogger(__name__)
 
-StringifyTransform = Callable[[Any], str]
-
-
-_whitespace_re = re.compile(r'(?u)\s+')
+_whitespace_re = re.compile(r'\s+')
 anon_identifier_re = re.compile(r'(@[a-zA-Z0-9_])[a-zA-Z0-9_]*\b')
-identifier_re = re.compile(r'''(?x)
+identifier_re = re.compile(r'''
     (   # This 'extends' _anon_identifier_re with the ordinary identifiers,
         # make sure they are in sync.
         (~?\b[a-zA-Z_])  # ordinary identifiers
     |   (@[a-zA-Z0-9_])  # our extension for names of anonymous entities
     )
     [a-zA-Z0-9_]*\b
-''')
+''', flags=re.VERBOSE)
 integer_literal_re = re.compile(r'[1-9][0-9]*(\'[0-9]+)*')
 octal_literal_re = re.compile(r'0[0-7]*(\'[0-7]+)*')
 hex_literal_re = re.compile(r'0[xX][0-9a-fA-F]+(\'[0-9a-fA-F]+)*')
 binary_literal_re = re.compile(r'0[bB][01]+(\'[01]+)*')
-integers_literal_suffix_re = re.compile(r'''(?x)
+integers_literal_suffix_re = re.compile(r'''
     # unsigned and/or (long) long, in any order, but at least one of them
     (
         ([uU]    ([lL]  |  (ll)  |  (LL))?)
@@ -39,8 +46,8 @@ integers_literal_suffix_re = re.compile(r'''(?x)
     )\b
     # the ending word boundary is important for distinguishing
     # between suffixes and UDLs in C++
-''')
-float_literal_re = re.compile(r'''(?x)
+''', flags=re.VERBOSE)
+float_literal_re = re.compile(r'''
     [+-]?(
     # decimal
       ([0-9]+(\'[0-9]+)*[eE][+-]?[0-9]+(\'[0-9]+)*)
@@ -52,10 +59,10 @@ float_literal_re = re.compile(r'''(?x)
         [0-9a-fA-F]+(\'[0-9a-fA-F]+)*([pP][+-]?[0-9a-fA-F]+(\'[0-9a-fA-F]+)*)?)
     | (0[xX][0-9a-fA-F]+(\'[0-9a-fA-F]+)*\.([pP][+-]?[0-9a-fA-F]+(\'[0-9a-fA-F]+)*)?)
     )
-''')
+''', flags=re.VERBOSE)
 float_literal_suffix_re = re.compile(r'[fFlL]\b')
 # the ending word boundary is important for distinguishing between suffixes and UDLs in C++
-char_literal_re = re.compile(r'''(?x)
+char_literal_re = re.compile(r'''
     ((?:u8)|u|U|L)?
     '(
       (?:[^\\'])
@@ -67,7 +74,7 @@ char_literal_re = re.compile(r'''(?x)
       | (?:U[0-9a-fA-F]{8})
       ))
     )'
-''')
+''', flags=re.VERBOSE)
 
 
 def verify_description_mode(mode: str) -> None:
@@ -81,18 +88,13 @@ class NoOldIdError(Exception):
 
 
 class ASTBaseBase:
-    def __eq__(self, other: Any) -> bool:
+    def __eq__(self, other: object) -> bool:
         if type(self) is not type(other):
-            return False
+            return NotImplemented
         try:
-            for key, value in self.__dict__.items():
-                if value != getattr(other, key):
-                    return False
+            return self.__dict__ == other.__dict__
         except AttributeError:
             return False
-        return True
-
-    __hash__: Callable[[], int] = None
 
     def clone(self) -> Any:
         return deepcopy(self)
@@ -101,13 +103,13 @@ class ASTBaseBase:
         raise NotImplementedError(repr(self))
 
     def __str__(self) -> str:
-        return self._stringify(lambda ast: str(ast))
+        return self._stringify(str)
 
     def get_display_string(self) -> str:
         return self._stringify(lambda ast: ast.get_display_string())
 
     def __repr__(self) -> str:
-        return '<%s>' % self.__class__.__name__
+        return f'<{self.__class__.__name__}: {self._stringify(repr)}>'
 
 
 ################################################################################
@@ -123,8 +125,16 @@ class ASTCPPAttribute(ASTAttribute):
     def __init__(self, arg: str) -> None:
         self.arg = arg
 
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, ASTCPPAttribute):
+            return NotImplemented
+        return self.arg == other.arg
+
+    def __hash__(self) -> int:
+        return hash(self.arg)
+
     def _stringify(self, transform: StringifyTransform) -> str:
-        return "[[" + self.arg + "]]"
+        return f"[[{self.arg}]]"
 
     def describe_signature(self, signode: TextElement) -> None:
         signode.append(addnodes.desc_sig_punctuation('[[', '[['))
@@ -133,35 +143,42 @@ class ASTCPPAttribute(ASTAttribute):
 
 
 class ASTGnuAttribute(ASTBaseBase):
-    def __init__(self, name: str, args: Optional["ASTBaseParenExprList"]) -> None:
+    def __init__(self, name: str, args: ASTBaseParenExprList | None) -> None:
         self.name = name
         self.args = args
 
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, ASTGnuAttribute):
+            return NotImplemented
+        return self.name == other.name and self.args == other.args
+
+    def __hash__(self) -> int:
+        return hash((self.name, self.args))
+
     def _stringify(self, transform: StringifyTransform) -> str:
-        res = [self.name]
         if self.args:
-            res.append(transform(self.args))
-        return ''.join(res)
+            return self.name + transform(self.args)
+        return self.name
 
 
 class ASTGnuAttributeList(ASTAttribute):
-    def __init__(self, attrs: List[ASTGnuAttribute]) -> None:
+    def __init__(self, attrs: list[ASTGnuAttribute]) -> None:
         self.attrs = attrs
 
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, ASTGnuAttributeList):
+            return NotImplemented
+        return self.attrs == other.attrs
+
+    def __hash__(self) -> int:
+        return hash(self.attrs)
+
     def _stringify(self, transform: StringifyTransform) -> str:
-        res = ['__attribute__((']
-        first = True
-        for attr in self.attrs:
-            if not first:
-                res.append(', ')
-            first = False
-            res.append(transform(attr))
-        res.append('))')
-        return ''.join(res)
+        attrs = ', '.join(map(transform, self.attrs))
+        return f'__attribute__(({attrs}))'
 
     def describe_signature(self, signode: TextElement) -> None:
-        txt = str(self)
-        signode.append(nodes.Text(txt))
+        signode.append(nodes.Text(str(self)))
 
 
 class ASTIdAttribute(ASTAttribute):
@@ -170,6 +187,14 @@ class ASTIdAttribute(ASTAttribute):
     def __init__(self, id: str) -> None:
         self.id = id
 
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, ASTIdAttribute):
+            return NotImplemented
+        return self.id == other.id
+
+    def __hash__(self) -> int:
+        return hash(self.id)
+
     def _stringify(self, transform: StringifyTransform) -> str:
         return self.id
 
@@ -184,26 +209,41 @@ class ASTParenAttribute(ASTAttribute):
         self.id = id
         self.arg = arg
 
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, ASTParenAttribute):
+            return NotImplemented
+        return self.id == other.id and self.arg == other.arg
+
+    def __hash__(self) -> int:
+        return hash((self.id, self.arg))
+
     def _stringify(self, transform: StringifyTransform) -> str:
-        return self.id + '(' + self.arg + ')'
+        return f'{self.id}({self.arg})'
 
     def describe_signature(self, signode: TextElement) -> None:
-        txt = str(self)
-        signode.append(nodes.Text(txt))
+        signode.append(nodes.Text(str(self)))
 
 
 class ASTAttributeList(ASTBaseBase):
-    def __init__(self, attrs: List[ASTAttribute]) -> None:
+    def __init__(self, attrs: list[ASTAttribute]) -> None:
         self.attrs = attrs
 
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, ASTAttributeList):
+            return NotImplemented
+        return self.attrs == other.attrs
+
+    def __hash__(self) -> int:
+        return hash(self.attrs)
+
     def __len__(self) -> int:
         return len(self.attrs)
 
-    def __add__(self, other: "ASTAttributeList") -> "ASTAttributeList":
+    def __add__(self, other: ASTAttributeList) -> ASTAttributeList:
         return ASTAttributeList(self.attrs + other.attrs)
 
     def _stringify(self, transform: StringifyTransform) -> str:
-        return ' '.join(transform(attr) for attr in self.attrs)
+        return ' '.join(map(transform, self.attrs))
 
     def describe_signature(self, signode: TextElement) -> None:
         if len(self.attrs) == 0:
@@ -234,22 +274,22 @@ class DefinitionError(Exception):
 
 class BaseParser:
     def __init__(self, definition: str, *,
-                 location: Union[nodes.Node, Tuple[str, int], str],
-                 config: "Config") -> None:
+                 location: nodes.Node | tuple[str, int] | str,
+                 config: Config) -> None:
         self.definition = definition.strip()
         self.location = location  # for warnings
         self.config = config
 
         self.pos = 0
         self.end = len(self.definition)
-        self.last_match: Match = None
-        self._previous_state: Tuple[int, Match] = (0, None)
-        self.otherErrors: List[DefinitionError] = []
+        self.last_match: re.Match[str] | None = None
+        self._previous_state: tuple[int, re.Match[str] | None] = (0, None)
+        self.otherErrors: list[DefinitionError] = []
 
         # in our tests the following is set to False to capture bad parsing
         self.allowFallbackExpressionParsing = True
 
-    def _make_multi_error(self, errors: List[Any], header: str) -> DefinitionError:
+    def _make_multi_error(self, errors: list[Any], header: str) -> DefinitionError:
         if len(errors) == 1:
             if len(header) > 0:
                 return DefinitionError(header + '\n' + str(errors[0][0]))
@@ -259,14 +299,11 @@ class BaseParser:
         for e in errors:
             if len(e[1]) > 0:
                 indent = '  '
-                result.append(e[1])
-                result.append(':\n')
+                result.extend((e[1], ':\n'))
                 for line in str(e[0]).split('\n'):
                     if len(line) == 0:
                         continue
-                    result.append(indent)
-                    result.append(line)
-                    result.append('\n')
+                    result.extend((indent, line, '\n'))
             else:
                 result.append(str(e[0]))
         return DefinitionError(''.join(result))
@@ -278,7 +315,7 @@ class BaseParser:
     def status(self, msg: str) -> None:
         # for debugging
         indicator = '-' * self.pos + '^'
-        print("%s\n%s\n%s" % (msg, self.definition, indicator))
+        logger.debug(f"{msg}\n{self.definition}\n{indicator}")  # NoQA: G004
 
     def fail(self, msg: str) -> None:
         errors = []
@@ -287,15 +324,14 @@ class BaseParser:
             'Invalid %s declaration: %s [error at %d]\n  %s\n  %s' %
             (self.language, msg, self.pos, self.definition, indicator))
         errors.append((exMain, "Main error"))
-        for err in self.otherErrors:
-            errors.append((err, "Potential other error"))
+        errors.extend((err, "Potential other error") for err in self.otherErrors)
         self.otherErrors = []
         raise self._make_multi_error(errors, '')
 
     def warn(self, msg: str) -> None:
         logger.warning(msg, location=self.location)
 
-    def match(self, regex: Pattern) -> bool:
+    def match(self, regex: re.Pattern[str]) -> bool:
         match = regex.match(self.definition, self.pos)
         if match is not None:
             self._previous_state = (self.pos, self.last_match)
@@ -344,8 +380,7 @@ class BaseParser:
     def matched_text(self) -> str:
         if self.last_match is not None:
             return self.last_match.group()
-        else:
-            return None
+        return ''
 
     def read_rest(self) -> str:
         rv = self.definition[self.pos:]
@@ -364,22 +399,22 @@ class BaseParser:
     ################################################################################
 
     @property
-    def id_attributes(self):
+    def id_attributes(self) -> Sequence[str]:
         raise NotImplementedError
 
     @property
-    def paren_attributes(self):
+    def paren_attributes(self) -> Sequence[str]:
         raise NotImplementedError
 
-    def _parse_balanced_token_seq(self, end: List[str]) -> str:
+    def _parse_balanced_token_seq(self, end: list[str]) -> str:
         # TODO: add handling of string literals and similar
         brackets = {'(': ')', '[': ']', '{': '}'}
         startPos = self.pos
-        symbols: List[str] = []
+        symbols: list[str] = []
         while not self.eof:
             if len(symbols) == 0 and self.current_char in end:
                 break
-            if self.current_char in brackets.keys():
+            if self.current_char in brackets:
                 symbols.append(brackets[self.current_char])
             elif len(symbols) > 0 and self.current_char == symbols[-1]:
                 symbols.pop()
@@ -391,7 +426,7 @@ class BaseParser:
                       % startPos)
         return self.definition[startPos:self.pos]
 
-    def _parse_attribute(self) -> Optional[ASTAttribute]:
+    def _parse_attribute(self) -> ASTAttribute | None:
         self.skip_ws()
         # try C++11 style
         startPos = self.pos
@@ -421,10 +456,9 @@ class BaseParser:
                     attrs.append(ASTGnuAttribute(name, exprs))
                 if self.skip_string_and_ws(','):
                     continue
-                elif self.skip_string_and_ws(')'):
+                if self.skip_string_and_ws(')'):
                     break
-                else:
-                    self.fail("Expected identifier, ')', or ',' in __attribute__.")
+                self.fail("Expected identifier, ')', or ',' in __attribute__.")
             if not self.skip_string_and_ws(')'):
                 self.fail("Expected ')' after '__attribute__((...)'")
             return ASTGnuAttributeList(attrs)
@@ -456,5 +490,5 @@ class BaseParser:
             res.append(attr)
         return ASTAttributeList(res)
 
-    def _parse_paren_expression_list(self) -> ASTBaseParenExprList:
+    def _parse_paren_expression_list(self) -> ASTBaseParenExprList | None:
         raise NotImplementedError
