@@ -1,13 +1,18 @@
 """Docutils-native XML and pseudo-XML writers."""
 
-from typing import Any
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, Any
 
 from docutils.writers.docutils_xml import Writer as BaseXMLWriter
 
-from sphinx.builders import Builder
+if TYPE_CHECKING:
+    from sphinx.builders import Builder
 
 
-class XMLWriter(BaseXMLWriter):
+class XMLWriter(BaseXMLWriter):  # type: ignore[misc]
+    output: str
+
     def __init__(self, builder: Builder) -> None:
         super().__init__()
         self.builder = builder
@@ -24,7 +29,7 @@ class XMLWriter(BaseXMLWriter):
         return super().translate()
 
 
-class PseudoXMLWriter(BaseXMLWriter):
+class PseudoXMLWriter(BaseXMLWriter):  # type: ignore[misc]
 
     supported = ('pprint', 'pformat', 'pseudoxml')
     """Formats this writer supports."""
@@ -32,7 +37,7 @@ class PseudoXMLWriter(BaseXMLWriter):
     config_section = 'pseudoxml writer'
     config_section_dependencies = ('writers',)
 
-    output = None
+    output: str
     """Final translated form of `document`."""
 
     def __init__(self, builder: Builder) -> None:
@@ -43,5 +48,5 @@ class PseudoXMLWriter(BaseXMLWriter):
         self.output = self.document.pformat()
 
     def supports(self, format: str) -> bool:
-        """This writer supports all format-specific elements."""
+        """All format-specific elements are supported."""
         return True
